/*  YAC: Yet Another Caller ID Program
 *  Copyright (C) 2002 Jensen Harris (jensen@sunflowerhead.com)
 *  http://www.sunflowerhead.com
 *
 *  YAC is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any
 *  later version.
 *
 *  YAC is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 *  details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with YAC; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  File: About.c
 *  This file contains code to display the About YAC dialog and to launch
 *  associated actions.
 */

#include "yac.h"

INT_PTR CALLBACK AboutDialogProc(HWND hwndDlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
	YACINFO *yi;

	GetYACInfoStruct(&yi);

	switch (msg)
	{
	case WM_INITDIALOG:
		InitAboutDialog(hwndDlg, yi);
		break;

	case WM_COMMAND:
		switch (LOWORD(wparam))
        {
		case IDC_GO:
			GoAboutCommand(hwndDlg, yi);
			break;

		case IDOK:
			yi->fAboutDlgShowing = FALSE;
			EndDialog(hwndDlg, TRUE);
			return TRUE;
		}
		break;

	case WM_DESTROY:
		if (yi->hfBold)
			DeleteObject(yi->hfBold);
		break;
	}

	return FALSE;
}


void InitAboutDialog(HWND hwnd, YACINFO *yi)
{
	HFONT hf;
	LOGFONT lf;
	HWND hwndItem;
	TCHAR szCaption[64], szText[256];

	// Make the title bold
	hf = GetWindowFont(GetDlgItem(hwnd, IDC_TITLE));
    GetObject(hf, sizeof(lf), &lf);
	lf.lfWeight = FW_BOLD;
	yi->hfBold = CreateFontIndirect(&lf);
	SetWindowFont(GetDlgItem(hwnd, IDC_TITLE), yi->hfBold, FALSE);

	// Fill up the "What do you want to do" dropdown
	hwndItem = GetDlgItem(hwnd, IDC_WHATTODO);

	LoadString(yi->hinst, IDS_READDOCS, szText, 256);
	SendMessage(hwndItem, CB_INSERTSTRING, -1, (LPARAM)szText);

	LoadString(yi->hinst, IDS_VISITWEB, szText, 256);
	SendMessage(hwndItem, CB_INSERTSTRING, -1, (LPARAM)szText);

	LoadString(yi->hinst, IDS_EMAIL, szText, 256);
	SendMessage(hwndItem, CB_INSERTSTRING, -1, (LPARAM)szText);

	LoadString(yi->hinst, IDS_GETHELP, szText, 256);
	SendMessage(hwndItem, CB_INSERTSTRING, -1, (LPARAM)szText);

	LoadString(yi->hinst, IDS_READLICENSE, szText, 256);
	SendMessage(hwndItem, CB_INSERTSTRING, -1, (LPARAM)szText);

	LoadString(yi->hinst, IDS_DLNEW, szText, 256);
	SendMessage(hwndItem, CB_INSERTSTRING, -1, (LPARAM)szText);

	LoadString(yi->hinst, IDS_DLLISTEN, szText, 256);
	SendMessage(hwndItem, CB_INSERTSTRING, -1, (LPARAM)szText);

	LoadString(yi->hinst, IDS_DLSOURCE, szText, 256);
	SendMessage(hwndItem, CB_INSERTSTRING, -1, (LPARAM)szText);

	LoadString(yi->hinst, IDS_LEARNYAK, szText, 256);
	SendMessage(hwndItem, CB_INSERTSTRING, -1, (LPARAM)szText);

	SendMessage(hwndItem, CB_SETCURSEL, 0, 0);

	// Set mode on About dialog
	if (yi->nMode == MODE_LISTENER)
	{
		LoadString(yi->hinst, IDS_RUNLISTENER, szText, 256);
		SetWindowText(GetDlgItem(hwnd, IDC_MODE), szText);
	}

	LoadString(yi->hinst, IDS_ABOUTYAC, szText, 256);
	_snprintf(szCaption, 64, szText, yi->szAppName);
	SetWindowText(hwnd, szCaption);
}


void GoAboutCommand(HWND hwnd, YACINFO *yi)
{
	HWND hwndItem;
	LRESULT lRet;
	TCHAR szPath[MAX_PATH];
	BOOL fShowError = FALSE;

	hwndItem = GetDlgItem(hwnd, IDC_WHATTODO);
	lRet = SendMessage(hwndItem, CB_GETCURSEL, 0, 0);

	switch (lRet)
	{
	case 0:
		if (GetProcessFolder(szPath))
		{
			strncat(szPath, "\\readme.txt", MAX_PATH);
			if (32 >= (INT_PTR)ShellExecute(hwnd, "open", szPath, NULL,
				NULL, SW_SHOWNORMAL))
			{
				fShowError = TRUE;
			}
		}
		else
			fShowError = TRUE;

		if (fShowError)
		{
			LoadString(yi->hinst, IDS_NOREADME, szPath, MAX_PATH);
			MessageBox(hwnd, szPath, yi->szAppName, MB_ICONSTOP);
		}
		break;

	case 1:
		ShellExecute(hwnd, "open", URL_HOMEPAGE, NULL, NULL, SW_SHOWNORMAL);
		break;
	case 2:
		ShellExecute(hwnd, "open", URL_EMAIL, NULL, NULL, SW_SHOWNORMAL);
		break;
	case 3:
		ShellExecute(hwnd, "open", URL_SUPPORT, NULL, NULL, SW_SHOWNORMAL);
		break;
	case 4:
		ShellExecute(hwnd, "open", URL_LICENSE, NULL, NULL, SW_SHOWNORMAL);
		break;
	case 5:
		ShellExecute(hwnd, "open", URL_NEWEST, NULL, NULL, SW_SHOWNORMAL);
		break;
	case 6:
		ShellExecute(hwnd, "open", URL_LISTENERS, NULL, NULL, SW_SHOWNORMAL);
		break;
	case 7:
		ShellExecute(hwnd, "open", URL_SOURCE, NULL, NULL, SW_SHOWNORMAL);
		break;
	case 8:
		ShellExecute(hwnd, "open", URL_LEARNYAK, NULL, NULL, SW_SHOWNORMAL);
		break;
	}
}