/*  YAC: Yet Another Caller ID Program
 *  Copyright (C) 2002 Jensen Harris (jensen@sunflowerhead.com)
 *  http://www.sunflowerhead.com
 *
 *  YAC is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any
 *  later version.
 *
 *  YAC is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 *  details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with YAC; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  File: Broadcast.c
 *  This file contains code which notifies listeners of a message or incoming
 *  call so that they can display it.
 */

#include "yac.h"

unsigned __stdcall NetConnectThread(void* lpParameter)
{
	INT_PTR i;
	TCHAR szMessage[MAX_CHAR_SENT];
	int nMsgLen;
	unsigned long nRemoteAddress;
	SOCKET sd;
	const int kBufferSize = 1024;
	CRITICAL_SECTION cs;
	YACINFO *yi;

	GetYACInfoStruct(&yi);

	InitializeCriticalSection(&cs);

	i = (INT_PTR)lpParameter;

	// Grab local copy of format string
	EnterCriticalSection(&cs);
	strncpy(szMessage, yi->szString, MAX_CHAR_SENT);
	LeaveCriticalSection(&cs);

	nMsgLen = (int)strlen(szMessage);
	nRemoteAddress = LookupAddress(yi->szAddr[i]);

	if (nRemoteAddress != INADDR_NONE)
	{
		sd = EstablishConnection(nRemoteAddress, htons(yi->nServerPort));
		send(sd, szMessage, nMsgLen, 0);
		shutdown(sd, SD_SEND);
		closesocket(sd);
		sd = 0;
	}

	DeleteCriticalSection(&cs);
	return 0;
}


SOCKET EstablishConnection(unsigned long nRemoteAddr, unsigned short nPort)
{
	SOCKET sd;
	SOCKADDR_IN sinRemote;

	// Create a stream socket
    sd = socket(AF_INET, SOCK_STREAM, 0);
    if (sd != INVALID_SOCKET)
	{
        sinRemote.sin_family = AF_INET;
        sinRemote.sin_addr.s_addr = nRemoteAddr;
        sinRemote.sin_port = nPort;
        if (connect(sd, (SOCKADDR*)&sinRemote, sizeof(SOCKADDR_IN)) ==
                SOCKET_ERROR)
		{
            sd = INVALID_SOCKET;
        }
    }

    return sd;
}


unsigned long LookupAddress(const char* pcHost)
{
    u_long nRemoteAddr = inet_addr(pcHost);
    if (nRemoteAddr == INADDR_NONE)
	{
        // pcHost isn't a dotted IP, so resolve it through DNS
        struct hostent* pHE = gethostbyname(pcHost);
        
		if (pHE == 0)
            return INADDR_NONE;

        nRemoteAddr = *((u_long*)pHE->h_addr_list[0]);
    }

    return nRemoteAddr;
}