/*  YAC: Yet Another Caller ID Program
 *  Copyright (C) 2002 Jensen Harris (jensen@sunflowerhead.com)
 *  http://www.sunflowerhead.com
 *
 *  YAC is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any
 *  later version.
 *
 *  YAC is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 *  details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with YAC; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  File: CallerID.c
 *  The guts of the TAPI Caller ID implementation.
 */

#include "yac.h"

unsigned __stdcall PhoneListenThread(void* lpParameter)
{
	DWORD dwVersion, dwDev, dwDeviceUse, dwPriv;
	HLINE hline;
	LINEINITIALIZEEXPARAMS liparams;
	LINECALLPARAMS lcp;
	LINEMESSAGE lmsg;
	BYTE by[65536], byLDC[sizeof(LINEDEVCAPS) + 65536];
	LINECALLINFO* pLCInfo = (LINECALLINFO*)by;
	LINEDEVCAPS* pLDC = (LINEDEVCAPS*)byLDC;
	LONG lError;
	INT_PTR nRet;
	BOOL fAlwaysUse = FALSE, fStartupSelect = FALSE, fRet;
	TCHAR szRegDevice[256];
	YACINFO *yi;
	CRITICAL_SECTION cs;
	TCHAR szName[32], szNumber[32];

	GetYACInfoStruct(&yi);

	// Support TAPI 2.0
	dwVersion = TAPI_CURRENT_VERSION;

	ZeroMemory(&liparams, sizeof(liparams));

	liparams.dwTotalSize = sizeof(LINEINITIALIZEEXPARAMS);
	liparams.dwNeededSize = sizeof(LINEINITIALIZEEXPARAMS);
	liparams.dwUsedSize = sizeof(LINEINITIALIZEEXPARAMS);
	liparams.dwOptions = LINEINITIALIZEEXOPTION_USEEVENT;

	if (!lineInitializeEx(&yi->hlapp, NULL, NULL, YAC_LINE_IDENT,
		&yi->dwNumDevs, &dwVersion, &liparams))
	{
		OutputDebugString("lineInitializeEx succeeded!\n");
	}
	else
	{
		NotifyError(IDS_LINEINIT, FALSE, yi->hinst, yi);
		PostMessage(yi->hwnd, YAC_MSG_QUIT, 0, 0);
		return -1;
	}

	if (yi->dwNumDevs > MAX_DEVICES)
		yi->dwNumDevs = MAX_DEVICES;

	// Let's figure out which device would be appropriate to use
	dwDeviceUse = 1000;
	fRet = GetInfo("StartupSelectDevice", szRegDevice, HKEY_LOCAL_MACHINE);

	// If select device on startup not selected, read in the devices
	// and preferred device
	fStartupSelect = TRUE;
	if (fRet && (atoi(szRegDevice) == 0))
		fStartupSelect = FALSE;

	GetInfo("Device", szRegDevice, HKEY_LOCAL_MACHINE);

	for (dwDev = 0; dwDev < yi->dwNumDevs; dwDev++)
	{
		pLDC->dwTotalSize = sizeof(LINEDEVCAPS) + 65536;
		if (0 == lineGetDevCaps(yi->hlapp, dwDev, dwVersion, 0, pLDC))
		{
			if (pLDC->dwLineNameSize)
			{
				strncpy(yi->DevInfo[dwDev].szDevice,
					(LPCTSTR)pLDC + pLDC->dwLineNameOffset, 64);

				// See if this is "the" device stored in the registry
				if (!lstrcmpi(yi->DevInfo[dwDev].szDevice, szRegDevice))
					dwDeviceUse = dwDev;
			}
		}
	}

	// If the startup select is on or the last device used no longer exists,
	// pop up the dialog
	if (dwDeviceUse == 1000 || fStartupSelect)
	{
		// Now, display the list of devices for the user to choose from
		yi->fDevicesDlgShowing = TRUE;

		nRet = DialogBox(yi->hinst, MAKEINTRESOURCE(IDD_DEVICES),
			yi->hwnd, DevicesDialogProc);

		// We return the device + 1000 to indicate that the "always use"
		// checkbox is ON.
		if (nRet >= 1000)
		{
			fAlwaysUse = TRUE;
			SaveInfo("StartupSelectDevice", "0", HKEY_LOCAL_MACHINE);
			yi->fStartupSelectDevice = FALSE;
			nRet -= 1000;
		}
		else
		{
			SaveInfo("StartupSelectDevice", "1", HKEY_LOCAL_MACHINE);
			yi->fStartupSelectDevice = TRUE;
		}
	
		for (dwDev = 0; dwDev < yi->dwNumDevs; dwDev++)
		{
			if (yi->DevInfo[dwDev].nListBoxID == nRet)
				dwDeviceUse = dwDev;
		}

		// Save out the "always use" value if appropriate
		if (fAlwaysUse)
			SaveInfo("Device", yi->DevInfo[dwDeviceUse].szDevice, HKEY_LOCAL_MACHINE);
	}

	// Get ready to open the line on the selected device
	ZeroMemory(&lcp, sizeof(lcp));

	lcp.dwTotalSize = sizeof(LINECALLPARAMS);
	lcp.dwBearerMode = LINEBEARERMODE_VOICE;
	lcp.dwMediaMode = LINEMEDIAMODE_DATAMODEM;

	dwPriv = yi->fShareModem ? LINECALLPRIVILEGE_MONITOR :
		LINECALLPRIVILEGE_OWNER;

	lError = lineOpen(yi->hlapp, dwDeviceUse, &hline, dwVersion, 0, 0,
		dwPriv, LINEMEDIAMODE_DATAMODEM, &lcp);

	if (!lError)
		OutputDebugString("lineOpen succeeded!\n");
	else
	{
		NotifyError(IDS_LINEOPEN, FALSE, yi->hinst, yi);

		// Next time, better prompt again
		SaveInfo("StartupSelectDevice", "1", HKEY_LOCAL_MACHINE);
		PostMessage(yi->hwnd, YAC_MSG_QUIT, 0, 0);
		return -1;
	}

	lineSetStatusMessages(hline, LINEDEVSTATE_RINGING, 0);

	// Here is the beginning of the main thread loop
	InitializeCriticalSection(&cs);

	while (TRUE)
	{
		lineGetMessage(yi->hlapp, &lmsg, INFINITE);

		if (WAIT_OBJECT_0 == WaitForSingleObject(yi->EventPhoneListenQuit, 0))
		{
			DeleteCriticalSection(&cs);
			return 0;
		}

		if (lmsg.dwMessageID == LINE_CALLSTATE)
		{
			if (lmsg.dwParam1 == LINECALLSTATE_IDLE)
			{
				lineDeallocateCall(lmsg.hDevice);
				
				szNumber[0] = '\0';
				szName[0] = '\0';
			}
		}

		else if (lmsg.dwMessageID == LINE_CALLINFO)
		{
			if (lmsg.dwParam1 == LINECALLINFOSTATE_CALLERID)
			{
				// Let's get the Caller ID information
				ZeroMemory(pLCInfo, 65536);
				pLCInfo->dwTotalSize = 65536;

				lineGetCallInfo(lmsg.hDevice, pLCInfo);

				if (pLCInfo->dwTotalSize < pLCInfo->dwNeededSize)
				{
					NotifyError(IDS_CALLINFOMEM, FALSE, yi->hinst, yi);
					PostMessage(yi->hwnd, YAC_MSG_QUIT, 0, 0);
					return -1;
				}

				// Does this actually have caller ID?
				if (pLCInfo->dwCallerIDFlags & LINECALLPARTYID_ADDRESS)
				{
					strncpy(szNumber, (LPCTSTR)pLCInfo + pLCInfo->dwCallerIDOffset, 32);
					MakeRawNumber(szNumber);
					EnterCriticalSection(&cs);
					strncpy(yi->szNumber, szNumber, 32);
					LeaveCriticalSection(&cs);
				}

				// How about a caller ID name?
				if (pLCInfo->dwCallerIDFlags & LINECALLPARTYID_NAME)
				{
					strncpy(szName, (LPCTSTR)pLCInfo + pLCInfo->dwCallerIDNameOffset, 32);
					EnterCriticalSection(&cs);
					strncpy(yi->szName, szName, 32);
					LeaveCriticalSection(&cs);
				}

				SendMessage(yi->hwnd, YAC_MSG_NOTIFY, 0, 0);
			}
		}
	}
}