/*  YAC: Yet Another Caller ID Program
 *  Copyright (C) 2002 Jensen Harris (jensen@sunflowerhead.com)
 *  http://www.sunflowerhead.com
 *
 *  YAC is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any
 *  later version.
 *
 *  YAC is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 *  details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with YAC; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  File: Formats.c
 *  This file contains code to format caller ID information nicely for
 *  different formats and locales.
 */

#include "yac.h"

void FormatNameAndNumber(TCHAR *szName, TCHAR *szNumber, YACINFO *yi)
{
	int i, j, nLastNonSpace;
	LONG lNameStatus = 0;
	TCHAR szFormat[64], szLen[8];

	// Strip blank characters after the name
	nLastNonSpace = -1; 

	for (i = 0; i < (int)strlen(szName); i++)
	{
		if (!isspace(szName[i]))
			nLastNonSpace = i;
	}

	szName[nLastNonSpace + 1] = '\0';

	// Check for Private Name, Out of Area, Unavailable
	if (szName[0] == 'P' && szName[1] == '\0')
	{
		lNameStatus = PRIVATE;
		LoadString(yi->hinst, IDS_PRIVATENAME, szName, MAX_CHAR_SENT);
	}
	else if (szName[0] == 'O' && szName[1] == '\0')
	{
		lNameStatus = OUTOFAREA;
		LoadString(yi->hinst, IDS_OUTOFAREANAME, szName, MAX_CHAR_SENT);
	}
	else if ((szName[0] == 'U' && szName[1] == '\0') ||
		(szName[0] == '\0'))
	{
		lNameStatus = UNAVAILABLE;
		LoadString(yi->hinst, IDS_UNAVAILNAME, szName, MAX_CHAR_SENT);
	}

	// Now, format the phone number based on the user's settings
	itoa((int)strlen(szNumber), szLen, 10);
	GetPrivateProfileString(INI_FORMATS, szLen, NONESTRING, szFormat,
		64, yi->szIniFile);

	if (StrCmp(NONESTRING, szFormat))
	{
		j = 0;

		for (i = 0; i < (int)strlen(szFormat); i++)
		{
			if (szFormat[i] == '#')
			{
				szFormat[i] = szNumber[j];
				j++;
				if (szNumber[j] == '\0')
					break;
			}
		}
		
		strncpy(szNumber, szFormat, 32);
	}

	// Now do number string substitution
	if (strlen(szNumber) <= 1)
	{
		if (lNameStatus == UNAVAILABLE)
			LoadString(yi->hinst, IDS_UNAVAILNUM, szNumber, MAX_CHAR_SENT);

		else if (lNameStatus == OUTOFAREA)
			LoadString(yi->hinst, IDS_OUTOFAREANUM, szNumber, MAX_CHAR_SENT);

		else if (lNameStatus == PRIVATE)
			LoadString(yi->hinst, IDS_PRIVATENUM, szNumber, MAX_CHAR_SENT);

		else
		{
			if (szNumber[0] == 'P' && szNumber[1] == '\0')
				LoadString(yi->hinst, IDS_PRIVATENUM, szNumber, MAX_CHAR_SENT);

			else if (szNumber[0] == 'O' && szNumber[1] == '\0')
				LoadString(yi->hinst, IDS_OUTOFAREANUM, szNumber, MAX_CHAR_SENT);

			else if ((szNumber[0] == 'U' && szNumber[1] == '\0') ||
				(szNumber[0] == '\0'))
			{
				LoadString(yi->hinst, IDS_UNAVAILNUM, szNumber, MAX_CHAR_SENT);
			}
		}
	}
}


void MakeRawNumber(TCHAR *szNumber)
{
	int i, j;

	j = 0;

	for (i = 0; i < (int)strlen(szNumber); i++)
	{
		if (isdigit(szNumber[i]))
		{
			szNumber[j] = szNumber[i];
			j++;
		}
	}
	
	szNumber[j] = '\0';
}


INT_PTR CALLBACK FormatsDialogProc(HWND hwndDlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
	YACINFO *yi;

	GetYACInfoStruct(&yi);

	switch (msg)
	{
	case WM_INITDIALOG:
		InitFormatsDialog(hwndDlg, yi);
		return TRUE;
		
	case WM_NOTIFY:
		if (((NM_LISTVIEW *)lparam)->hdr.code == LVN_ITEMCHANGED)
		{
			if (((NM_LISTVIEW *)lparam)->uNewState & LVIS_SELECTED)
				DoFormatsSelectionChanged(hwndDlg);
		}
		return 0;

	case WM_COMMAND:
		switch (LOWORD(wparam))
        {
		case IDOK:
			SaveFormatsData(hwndDlg, yi);

		// Intentional fallthrough
		case IDCANCEL:
			yi->fFormatsDlgShowing = FALSE;
			EndDialog(hwndDlg, TRUE);
			return TRUE;

		case IDC_EDIT_FORMAT:
			if ((HIWORD(wparam)) == EN_CHANGE)
				DoFormatsEditChanged(hwndDlg);
			return 0;
		}
		break;
	}

	return FALSE;
}


BOOL InitFormatsDialog(HWND hwnd, YACINFO *yi)
{
	TCHAR szColumn[64], szFormat[64];
	TCHAR szKey[8];
	LVCOLUMN lvc;
	LVITEM lvi;
	HWND hwndList;
	RECT rc;
	int nIndex, i;

	ZeroMemory(&lvc, sizeof(lvc));
	ZeroMemory(&lvi, sizeof(lvi));

	// Finally, we have enough memory to hold the whole string--party on it.
	hwndList = GetDlgItem(hwnd, IDC_LISTFORMATS);
	ListView_SetImageList(hwndList, yi->im, LVSIL_SMALL);
	ListView_SetExtendedListViewStyle(hwndList, LVS_EX_FULLROWSELECT);

	lvc.fmt = LVCFMT_LEFT;
	lvc.cx = NUMBER_COLUMN_SIZE;
	LoadString(yi->hinst, IDS_COLUMNDIGITS, szColumn, 64);
	lvc.pszText = szColumn;
	lvc.cchTextMax = 64;
	lvc.mask = LVCF_FMT | LVCF_TEXT | LVCF_WIDTH;
	ListView_InsertColumn(hwndList, 0, &lvc);

	LoadString(yi->hinst, IDS_COLUMNFORMAT, szColumn, 64);
	GetClientRect(hwndList, &rc);
	lvc.cx = rc.right - rc.left - GetSystemMetrics(SM_CXVSCROLL) - NUMBER_COLUMN_SIZE;

	ListView_InsertColumn(hwndList, 1, &lvc);

	// Now, we need to add the items to the list
	lvi.mask = LVIF_IMAGE | LVIF_TEXT;
	lvi.iImage = 1;
	lvi.cchTextMax = 64;

	for (i = 1; i <= SUPPORTED_FORMATS; i++)
	{
		itoa(i, szKey, 10);

		GetPrivateProfileString(INI_FORMATS, szKey, NONESTRING,
			szFormat, 64, yi->szIniFile);

		if (!strcmp(szFormat, NONESTRING))
			szFormat[0] = '\0';

		// Add the row
		lvi.iItem = i;
		lvi.iSubItem = 0;
		lvi.pszText = szKey;

		nIndex = ListView_InsertItem(hwndList, &lvi);

		lvi.iItem = nIndex;
		lvi.iSubItem = 1;
		lvi.pszText = szFormat;
		ListView_SetItem(hwndList, &lvi);
	}

	// Select the top item in the listview
	ListView_SetItemState(hwndList, 0,
		LVIS_SELECTED | LVIS_FOCUSED, LVIS_SELECTED | LVIS_FOCUSED);
	SetFocus(hwndList);

	return TRUE;
}


void DoFormatsEditChanged(HWND hwnd)
{
	HWND hwndEdit, hwndList;
	TCHAR szText[64];
	LVITEM lvi;
	int nSel;

	ZeroMemory(&lvi, sizeof(lvi));

	hwndList = GetDlgItem(hwnd, IDC_LISTFORMATS);
	hwndEdit = GetDlgItem(hwnd, IDC_EDIT_FORMAT);
	GetWindowText(hwndEdit, szText, 64);

	// Sync up edit control with listview
	nSel = ListView_GetNextItem(hwndList, -1, LVNI_SELECTED);	

	lvi.mask = LVIF_TEXT;
	lvi.iItem = nSel;
	lvi.cchTextMax = 64;
	lvi.iSubItem = 1;
	lvi.pszText = szText;
	ListView_SetItem(hwndList, &lvi);

	// Fill in example text
	CalcExampleText(szText, nSel + 1);
	SetWindowText(GetDlgItem(hwnd, IDC_EXAMPLE), szText);
}


void DoFormatsSelectionChanged(HWND hwnd)
{
	int nSel;
	TCHAR szText[64];
	HWND hwndList, hwndEdit;

	hwndList = GetDlgItem(hwnd, IDC_LISTFORMATS);
	hwndEdit = GetDlgItem(hwnd, IDC_EDIT_FORMAT);

	nSel = ListView_GetNextItem(hwndList, -1, LVNI_SELECTED);

	ListView_GetItemText(hwndList, nSel, 1, szText, 64);
	SetWindowText(hwndEdit, szText);

	// Fill in the digit text
	itoa(nSel + 1, szText, 10);
	SetWindowText(GetDlgItem(hwnd, IDC_NUMDIGITS), szText);
}


void SaveFormatsData(HWND hwnd, YACINFO *yi)
{
	HWND hwndList;
	int i;
	TCHAR szText[64], szKey[8];

	hwndList = GetDlgItem(hwnd, IDC_LISTFORMATS);

	for (i = 0; i < SUPPORTED_FORMATS; i++)
	{
		itoa(i + 1, szKey, 10);
		ListView_GetItemText(hwndList, i, 1, szText, 64);
	
		if (szText[0] == '\0')
		{
			// A format is being deleted
			WritePrivateProfileString(INI_FORMATS, szKey, NULL, yi->szIniFile);
		}
		else
			WritePrivateProfileString(INI_FORMATS, szKey, szText, yi->szIniFile);
	}
}


void CalcExampleText(TCHAR *szText, int nNumDigits)
{
	int i, nCounted;
	TCHAR szTemp[64];

	nCounted = 0;

	if (szText[0] == '\0')
	{
		for (i = 0; i < nNumDigits; i++)
			szText[i] = GetRandomNumChar(i);

		szText[i] = '\0';
		return;
	}

	// We do have some kind of formatting string
	for (i = 0; i < (int)strlen(szText); i++)
	{
		if ((szText[i] == '#') && (nCounted < nNumDigits))
		{
			szTemp[i] = GetRandomNumChar(nCounted);
			nCounted++;
		}
		else
			szTemp[i] = szText[i];
	}
	
	szTemp[i] = '\0';

	strcpy(szText, szTemp);
}