/*  YAC: Yet Another Caller ID Program
 *  Copyright (C) 2002 Jensen Harris (jensen@sunflowerhead.com)
 *  http://www.sunflowerhead.com
 *
 *  YAC is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any
 *  later version.
 *
 *  YAC is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 *  details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with YAC; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  File: Listener.c
 *  This file contains the code which makes up the network listener part of
 *  YAC.  Mostly this is networking code.
 */

#include "yac.h"

unsigned __stdcall ListenThread(void* lpParameter)
{
	SOCKET clientFd = 0;
	int serverLen, clientLen;
	struct sockaddr_in serverINETAddress;
	struct sockaddr* serverSockAddrPtr;
	struct sockaddr_in clientINETAddress;
	struct sockaddr* clientSockAddrPtr;
	int on = 1;
	TCHAR szInput[MAX_CHAR_SENT];
	YACINFO *yi;

	GetYACInfoStruct(&yi);

	yi->serverFd = 0;

	while (TRUE)
	{
		yi->serverFd = socket(AF_INET, SOCK_STREAM, 0);
		serverLen = sizeof(serverINETAddress);
		setsockopt(yi->serverFd, SOL_SOCKET, SO_REUSEADDR, (void *)&on, sizeof(on));

		ZeroMemory(&serverINETAddress, serverLen);
		serverINETAddress.sin_family = AF_INET;
		serverINETAddress.sin_addr.s_addr = htonl(INADDR_ANY);
		serverINETAddress.sin_port = htons(yi->nServerPort);

		clientLen = sizeof(clientINETAddress);
		ZeroMemory(&clientINETAddress, clientLen);

		serverSockAddrPtr = (struct sockaddr*) &serverINETAddress;
		bind(yi->serverFd, serverSockAddrPtr, serverLen);

		listen(yi->serverFd, 1);

		clientSockAddrPtr = (struct sockaddr*) &clientINETAddress;
		clientFd = accept(yi->serverFd, clientSockAddrPtr, &clientLen);

		if (WAIT_OBJECT_0 == WaitForSingleObject(yi->EventListenQuit, 0))
			goto cleanup;

		ZeroMemory(szInput, sizeof(szInput));
		ReadLineFromSocket(clientFd, szInput);

		closesocket(clientFd);
		clientFd = 0;

		if (szInput[0] != '\0')
		{
			// Shorten to maximum length
			szInput[MSG_LIMIT_CHARS] = '\0';

			// Notify the user of the text that came in
			SendMessage(yi->hwnd, YAC_MSG_NOTIFYUSER, (WPARAM)szInput, 0);
		}

		closesocket(yi->serverFd);
		yi->serverFd = 0;
	}

cleanup:
	if (yi->serverFd)
		closesocket(yi->serverFd);

    if (clientFd)
		closesocket(clientFd);

	return 0;
}


void ReadLineFromSocket(SOCKET fd, char *szText)
{
	int ch, i;

    // No need to read in anything longer than MAX_CHAR_SENT no matter what.
    for (i = 0; i < MAX_CHAR_SENT; i++)
    {
    	ch = recv(fd, szText, 1, 0);
        szText++;

        if (ch == 0)
        	goto cleanup;
    }

cleanup:
	szText[i] = '\0';
}