/*  YAC: Yet Another Caller ID Program
 *  Copyright (C) 2002 Jensen Harris (jensen@sunflowerhead.com)
 *  http://www.sunflowerhead.com
 *
 *  YAC is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any
 *  later version.
 *
 *  YAC is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 *  details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with YAC; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  File: ListenersDlg.c
 *  This file contains code to manage the Select Listeners dialog, as well
 *  as to load and save the listeners list out from and to disk.
 */

#include "yac.h"

INT_PTR CALLBACK ListenersDialogProc(HWND hwndDlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
	YACINFO *yi;

	GetYACInfoStruct(&yi);

	switch (msg)
	{
	case WM_INITDIALOG:
		InitListenersDialog(hwndDlg, yi);
		break;

	case WM_COMMAND:
		switch (LOWORD(wparam))
        {
		case IDC_TEST:
			TestListeners(hwndDlg, yi);
			break;

		case IDOK:
			SaveListenersData(hwndDlg, yi);
			ReadListenersData(yi);

		// intentional fallthrough
		case IDCANCEL:
			yi->fListenersDlgShowing = FALSE;
			EndDialog(hwndDlg, TRUE);
			return TRUE;
		}
	}

	return FALSE;
}


void TestListeners(HWND hwnd, YACINFO *yi)
{
	TCHAR szText[256];
	CRITICAL_SECTION cs;

	InitializeCriticalSection(&cs);
	LoadString(yi->hinst, IDS_TESTLISTENERS, szText, 256);
	
	if (IDYES == MessageBox(hwnd, szText, yi->szAppName,
		MB_YESNO | MB_ICONINFORMATION))
	{
		// Send a message to all the listeners.
		SaveListenersData(hwnd, yi);
		ReadListenersData(yi);

		EnterCriticalSection(&cs);
		strcpy(yi->szName, "YAC Test Call");
		strcpy(yi->szNumber, "4255551212");
		LeaveCriticalSection(&cs);

		NotifyClients(yi);
		DeleteCriticalSection(&cs);
	}
}


void InitListenersDialog(HWND hwnd, YACINFO *yi)
{
	int i;
	HWND hwndItem;
	
	// Fill in the listeners UI from our internal structures
	for (i = 0; i < yi->nNumListeners; i++)
	{
		// Now, do the edit box
		hwndItem = GetDlgItem(hwnd, i + IDC_ADDR_FIRST);
		SetWindowText(hwndItem, yi->szAddr[i]);
	}
}


void SaveListenersData(HWND hwnd, YACINFO *yi)
{
	int i, nListeners = 0;
	HWND hwndItem;
	TCHAR szAddr[256], szString[32];

	for (i = 0; i < 10; i++)
	{
		// Check the address
		hwndItem = GetDlgItem(hwnd, i + IDC_ADDR_FIRST);
		GetWindowText(hwndItem, szAddr, 256);

		if (szAddr[0] != '\0')
		{
			// Write out data to the config file
			sprintf(szString, "Address%d", nListeners);
			WritePrivateProfileString(INI_LISTENERS, szString,
				szAddr, yi->szIniFile);

			nListeners++;
		}
	}

	// Now, write out to the file any unused sections
	for (i = nListeners; i < 10; i++)
	{
		sprintf(szString, "Address%d", i);
		WritePrivateProfileString(INI_LISTENERS, szString,
			"", yi->szIniFile);
	}
}


void ReadListenersData(YACINFO *yi)
{
	int i;
	TCHAR szAddress[32], szInAddr[256];

	// Now, we need to read data in and set UI accordingly.
	yi->nNumListeners = 0;

	for (i = 0; i < 10; i++)
	{
		sprintf(szAddress, "Address%d", i);

		GetPrivateProfileString(INI_LISTENERS, szAddress, NONESTRING,
			szInAddr, 256, yi->szIniFile);

		if (lstrcmp(szInAddr, NONESTRING))
		{
			strncpy(yi->szAddr[yi->nNumListeners], szInAddr, 256);
			yi->nNumListeners++;
		}
	}
}