/*  YAC: Yet Another Caller ID Program
 *  Copyright (C) 2002 Jensen Harris (jensen@sunflowerhead.com)
 *  http://www.sunflowerhead.com
 *
 *  YAC is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any
 *  later version.
 *
 *  YAC is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 *  details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with YAC; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  File: NameSubs.c
 *  In this file are all routines dealing with name substitutions, including
 *  the UI for adding and removing substitutions.
 */

#include "yac.h"

BOOL InitSubsDialog(HWND hwnd, YACINFO *yi)
{
	TCHAR *szKeys, *szOrig;
	TCHAR szColumn[64], szName[64];
	DWORD dwSize = 256, dwRet;
	LVCOLUMN lvc;
	LVITEM lvi;
	HWND hwndList;
	RECT rc;
	int nItem = 0, nIndex;

	ZeroMemory(&lvc, sizeof(lvc));
	ZeroMemory(&lvi, sizeof(lvi));

	szKeys = (TCHAR *)HeapAlloc(GetProcessHeap(),
		HEAP_ZERO_MEMORY, dwSize);

	if (!szKeys)
		return FALSE;

	dwRet = GetPrivateProfileString(INI_SUBS, NULL,
		NONESTRING, szKeys, dwSize, yi->szIniFile);

	while (dwRet == (dwSize - 2))
	{
		dwSize *= 2;

		szKeys = (TCHAR *)HeapReAlloc(GetProcessHeap(),
			HEAP_ZERO_MEMORY, szKeys, dwSize);

		if (!szKeys)
			return FALSE;
		
		dwRet = GetPrivateProfileString(INI_SUBS, NULL,
			NONESTRING, szKeys, dwSize, yi->szIniFile);
	}

	// Finally, we have enough memory to hold the whole string--party on it.
	szOrig = szKeys;

	hwndList = GetDlgItem(hwnd, IDC_NAMELIST);
	ListView_SetImageList(hwndList, yi->im, LVSIL_SMALL);
	ListView_SetExtendedListViewStyle(hwndList, LVS_EX_FULLROWSELECT);

	lvc.fmt = LVCFMT_LEFT;
	lvc.cx = PHONE_COLUMN_SIZE;
	LoadString(yi->hinst, IDS_COLUMNPHONE, szColumn, 64);
	lvc.pszText = szColumn;
	lvc.cchTextMax = 64;
	lvc.mask = LVCF_FMT | LVCF_TEXT | LVCF_WIDTH;
	ListView_InsertColumn(hwndList, 0, &lvc);

	LoadString(yi->hinst, IDS_COLUMNNAME, szColumn, 64);
	GetClientRect(hwndList, &rc);
	lvc.cx = rc.right - rc.left - GetSystemMetrics(SM_CXVSCROLL) - PHONE_COLUMN_SIZE;

	ListView_InsertColumn(hwndList, 1, &lvc);

	// Now, we need to add the items to the list
	lvi.mask = LVIF_IMAGE | LVIF_TEXT;
	lvi.iImage = 0;
	lvi.cchTextMax = 64;

	while (szKeys[0] != '\0')
	{
		lvi.iItem = nItem;
		lvi.iSubItem = 0;
		lvi.pszText = szKeys;

		GetPrivateProfileString(INI_SUBS, szKeys, NONESTRING,
			szName, 64, yi->szIniFile);

		if (lstrcmp(szName, NONESTRING))
		{
			nIndex = ListView_InsertItem(hwndList, &lvi);

			lvi.iItem = nIndex;
			lvi.iSubItem = 1;
			lvi.pszText = szName;
			ListView_SetItem(hwndList, &lvi);
		}

		szKeys += strlen(szKeys) + 1;
		nItem++;
	}

	if (szOrig)
		HeapFree(GetProcessHeap(), 0, szOrig);

	return TRUE;
}


INT_PTR CALLBACK SubsDialogProc(HWND hwndDlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
	YACINFO *yi;

	GetYACInfoStruct(&yi);

	switch (msg)
	{
	case WM_INITDIALOG:
		InitSubsDialog(hwndDlg, yi);
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wparam))
        {
		case IDC_REMOVE:
			RemoveSelectedItem(hwndDlg, yi);
			break;

		case IDC_ADD:
			AddNewItem(hwndDlg, yi);
			break;

		case IDOK:
			yi->fSubsDlgShowing = FALSE;
			EndDialog(hwndDlg, TRUE);
			return TRUE;
		}
		break;
	}

	return FALSE;
}


void AddNewItem(HWND hwnd, YACINFO *yi)
{
	TCHAR szName[24], szNumber[24], szText[256];
	LVITEM lvi;
	HWND hwndList;
	int nRet;

	ZeroMemory(&lvi, sizeof(lvi));
	GetWindowText(GetDlgItem(hwnd, IDC_NAME), szName, 24);
	GetWindowText(GetDlgItem(hwnd, IDC_NUMBER), szNumber, 24);

	if (szName[0] == '\0' || szNumber[0] == '\0')
	{
		LoadString(yi->hinst, IDS_CANTADD, szText, 256);
		MessageBox(hwnd, szText, yi->szAppName, MB_ICONWARNING);
		return;
	}

	if (!WritePrivateProfileString(INI_SUBS, szNumber,
		szName, yi->szIniFile))
	{
		LoadString(yi->hinst, IDS_ADDFAILURE, szText, 256);
		MessageBox(hwnd, szText, yi->szAppName, MB_ICONSTOP);
		return;
	}

	// Now, add the item to the UI list
	hwndList = GetDlgItem(hwnd, IDC_NAMELIST);
	lvi.mask = LVIF_IMAGE | LVIF_TEXT;
	lvi.iImage = 0;
	lvi.cchTextMax = 64;
	lvi.iItem = 0;
	lvi.iSubItem = 0;
	lvi.pszText = szNumber;

	nRet = ListView_InsertItem(hwndList, &lvi);
	
	lvi.iItem = nRet;
	lvi.iSubItem = 1;
	lvi.pszText = szName;
	ListView_SetItem(hwndList, &lvi);

	SetWindowText(GetDlgItem(hwnd, IDC_NAME), "");
	SetWindowText(GetDlgItem(hwnd, IDC_NUMBER), "");
	SetFocus(GetDlgItem(hwnd, IDC_NUMBER));
}


void RemoveSelectedItem(HWND hwnd, YACINFO *yi)
{
	HWND hwndList;
	int nRet;
	TCHAR szText[256];

	hwndList = GetDlgItem(hwnd, IDC_NAMELIST);
	nRet = ListView_GetSelectedCount(hwndList);

	if (!nRet)
	{
		LoadString(yi->hinst, IDS_NOSELECTION, szText, 256);
		MessageBox(hwnd, szText, yi->szAppName, MB_ICONWARNING);
		return;
	}

	nRet = ListView_GetNextItem(hwndList, -1, LVNI_SELECTED);
	ListView_GetItemText(hwndList, nRet, 0, szText, 256);
	WritePrivateProfileString(INI_SUBS, szText, NULL, yi->szIniFile);
	ListView_DeleteItem(hwndList, nRet);

	// Put focus back in the listview for niceness
	ListView_SetItemState(hwndList, 0,
		LVIS_SELECTED | LVIS_FOCUSED, LVIS_SELECTED | LVIS_FOCUSED);
	SetFocus(hwndList);
}


BOOL DoNameSubstitution(TCHAR *szName, TCHAR *szNumber, YACINFO *yi)
{
	TCHAR szNewName[64];

	GetPrivateProfileString(INI_SUBS, szNumber, NONESTRING,
		szNewName, 64, yi->szIniFile);

	if (!lstrcmp(szNewName, NONESTRING))
		return FALSE;

	strncpy(szName, szNewName, 31);
	
	return TRUE;
}