/*  YAC: Yet Another Caller ID Program
 *  Copyright (C) 2002 Jensen Harris (jensen@sunflowerhead.com)
 *  http://www.sunflowerhead.com
 *
 *  YAC is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any
 *  later version.
 *
 *  YAC is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 *  details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with YAC; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  File: NotifyUI.c
 *  This file contains code that deals with everything about displaying the UI
 *  and other appropriate actions when a notification is received.
 */

#include "yac.h"

TCHAR	g_szName[32];
TCHAR	g_szNumber[32];
TCHAR	g_szText[MAX_CHAR_SENT];
TCHAR	g_szNoCRLFText[MAX_CHAR_SENT];
HWND	g_hwnd;
BOOL	g_fCallerID;
BOOL	g_fInWindow;

BOOL NotifyUser(TCHAR *szInText, YACINFO *yi)
{
	TCHAR szTime[64], szString[256];
	TCHAR szName[MAX_CHAR_SENT], szNumber[MAX_CHAR_SENT];
	BOOL fCallerID;

	strncpy(g_szText, szInText, MAX_CHAR_SENT);
	strcpy(g_szNoCRLFText, g_szText);
	StripCRLF(g_szNoCRLFText);

	if (yi->fPlaySounds && !yi->fUseBalloons)
		PlaySound(yi->szIncomingSound, yi->hinst, SND_RESOURCE | SND_ASYNC);

	// Now, see if this is Caller ID or not
	fCallerID = DecodeCallerIDInfo(g_szText, szName, szNumber);

	if (fCallerID)
	{
		g_fCallerID = TRUE;

		// Truncate down to a predictable size
		strncpy(g_szName, szName, 32);
		g_szName[31] = '\0';
		strncpy(g_szNumber, szNumber, 32);
		g_szNumber[31] = '\0';

		WriteLogIncomingCall(g_szName, g_szNumber, yi);

		if (yi->fUseBalloons)
		{
			// Bad things happen with balloon and notify window up
			FindAndDestroyNotifyWindow();

			// Show the UI using Windows balloons
			yi->ni.uFlags = NIF_ICON | NIF_MESSAGE | NIF_INFO;
		
			LoadString(yi->hinst, IDS_BALLOON_FMT, szString, 256);
			sprintf(g_szText, szString, g_szName, g_szNumber);
			strcpy(yi->ni.szInfo, g_szText);

			GetTimeFormat(LOCALE_USER_DEFAULT, TIME_NOSECONDS, NULL, NULL, szTime, 64);
			LoadString(yi->hinst, IDS_PHONERINGING, szString, 256);
			sprintf(yi->ni.szInfoTitle, szString, szTime);

			Shell_NotifyIcon(NIM_MODIFY, &yi->ni);

			// Execute helper script (if it exists) - MattC 17-Apr-07
			ExecuteHelperScript();

			return FALSE;
		}
		else
		{
			// Show pretty UI
			g_hwnd = CreateNotifyWindow(yi);

			// Execute helper script (if it exists) - MattC 17-Apr-07
			ExecuteHelperScript();

			return TRUE;
		}
	}
	else
	{
		// Show the raw text message we received.
		if (yi->fWriteTextToLog)
			WriteLogIncomingMessage(g_szNoCRLFText, yi);

		g_fCallerID = FALSE;
		g_hwnd = CreateNotifyWindow(yi);

		// Execute helper script (if it exists) - MattC 17-Apr-07
		ExecuteHelperScript();

		return TRUE;
	}



	// MattC Addition


	return FALSE;
}



void ExecuteHelperScript()
{
	// ----------------------------------------------------
	// Matt Collinge :: www.mattcollinge.co.uk :: 17-Apr-07
	// ----------------------------------------------------
	// Call a helper script named yac-helper.cmd that
	// resides in the same folder as yac.exe. The script
	// will be passed parameters pertaining to the 
	// inbound call.
	// ----------------------------------------------------

	TCHAR szParam[256];
	TCHAR szParamMsg[256];
	TCHAR szPath[MAX_PATH];
	int iShellRet;

	if (GetProcessFolder(szPath)) {
		szParam[0] = '\0';
		if (g_fCallerID) {
			strcat(szParam, "call ");
		} else {
			strcat(szParam, "message ");
			strncpy(szParamMsg, g_szNoCRLFText, 255);
		}

		strcat(szParam, "\x22");
		strcat(szParam, g_szNumber);
		strcat(szParam, "\x22 \x22");
		strcat(szParam, g_szName);
		strncat(szParam, "\x22", 255);
		strncat(szPath, "\\yac-helper.cmd", MAX_PATH);

		// SW_HIDE
		// SW_SHOWNORMAL
		iShellRet = (int) ShellExecute(NULL, "open", szPath, szParam, NULL, SW_HIDE);
	}
}

void FindAndDestroyNotifyWindow()
{
	HWND hwnd;

	hwnd = FindWindow(YAC_NOTIFYWND_CLASS, NULL);

	if (hwnd)
		DestroyWindow(hwnd);
}


void StopNotifyingUser(YACINFO *yi)
{
	if (yi->fPlaySounds)
		PlaySound(yi->szDoneSound, yi->hinst, SND_RESOURCE | SND_ASYNC);

	if (g_hwnd)
	{
		DestroyWindow(g_hwnd);
		g_hwnd = NULL;
	}

	g_szName[0] = '\0';
	g_szNumber[0] = '\0';
	g_szText[0] = '\0';
}


HWND CreateNotifyWindow(YACINFO *yi)
{
	WNDCLASSEX wcex;
	TCHAR wname[32];
	HWND hwnd;
	RECT rc;
	DWORD dwStyleEx;
	HMODULE hmod;
	int x, y;
	SETLAYEREDWINDOWATTRIBUTESFUNC SetLayeredWindowAttributesFunc = NULL;

	FindAndDestroyNotifyWindow();
	strcpy(wname, YAC_NOTIFYWND_CLASS);

	wcex.cbSize = sizeof(WNDCLASSEX); 

	wcex.style			= 0;
	wcex.lpfnWndProc	= (WNDPROC)NotifyWndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= yi->hinst;
	wcex.hIcon			= NULL;
	wcex.hCursor		= NULL,
	wcex.hbrBackground	= NULL;
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= wname;
	wcex.hIconSm		= NULL;

	RegisterClassEx(&wcex);

	// Figure out where to place the window on the primary monitor
	SystemParametersInfo(SPI_GETWORKAREA, 0, &rc, 0);

	// Get the right Ex styles
	dwStyleEx = WS_EX_TOPMOST | WS_EX_TOOLWINDOW;

	if (yi->fTranslucentWindows)
		dwStyleEx |= WS_EX_LAYERED;

	// Get proper size for window
	x = NOTIFY_WND_X;
	y = NOTIFY_WND_Y;

	// Create the window
	hwnd = CreateWindowEx(dwStyleEx, wname, "", WS_POPUP, rc.right - x,
		rc.bottom - y, x, y, NULL, NULL, yi->hinst, NULL);

	if (!hwnd)
		return NULL;

	if (yi->fTranslucentWindows)
	{
		// The masses want the listener to work on Win98.
		hmod = GetModuleHandle("user32.dll");
		
		// Get address to SetLayeredWindowAttributes() so we run on Win98/Me
		SetLayeredWindowAttributesFunc = (SETLAYEREDWINDOWATTRIBUTESFUNC)
			GetProcAddress(hmod, "SetLayeredWindowAttributes");
	
		if (SetLayeredWindowAttributesFunc)
		{
			SetLayeredWindowAttributesFunc(hwnd, COLOR_TRANSPARENT,
				MENU_TRANSPARENCY, LWA_ALPHA | LWA_COLORKEY);
		}
	}

	ShowWindow(hwnd, SW_SHOWNORMAL);
	g_fInWindow = FALSE;
	return hwnd;
}


LRESULT CALLBACK NotifyWndProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
	HDC hdc;
	PAINTSTRUCT ps;
	YACINFO *yi;

	GetYACInfoStruct(&yi);

	switch (msg)
	{
	case WM_MOUSEMOVE:
		if (!g_fInWindow)
		{
			g_fInWindow = TRUE;
			SetUpMouseTracking(hwnd);

			if (g_fCallerID)
			{
				SetCursor(yi->hcArrow);
			}
			else
			{
				SetCursor(yi->hcHand);
				InvalidateRect(hwnd, NULL, FALSE);
			}
		}
		return 0;

	case WM_LBUTTONDOWN:
		if (!g_fCallerID)
			LaunchReadTextDialog(hwnd, yi);
		return 0;

	case WM_MOUSELEAVE:
		g_fInWindow = FALSE;
		SetCursor(yi->hcArrow);
		InvalidateRect(hwnd, NULL, FALSE);
		return 0;

	case WM_PAINT:
		hdc = BeginPaint(hwnd, &ps);
		PaintNotifyWindow(hwnd, hdc, yi);
		EndPaint(hwnd, &ps);
		return 0;

	default:
		return DefWindowProc(hwnd, msg, wparam, lparam);
	}
}


void SetUpMouseTracking(HWND hwnd)
{
	TRACKMOUSEEVENT tme;

	tme.cbSize = sizeof(TRACKMOUSEEVENT);
	tme.dwFlags = TME_LEAVE;
	tme.dwHoverTime = HOVER_DEFAULT;
	tme.hwndTrack = hwnd;

	TrackMouseEvent(&tme);
}


void PaintNotifyWindow(HWND hwnd, HDC hdc, YACINFO *yi)
{	
	HBRUSH hbrBlack, hbrOld;
	HPEN hpen, hpOld;
	RECT rc;
	HDC hdcBitmap;
	TCHAR szText[128], szTime[64], szString[256];
	HBITMAP hbmOld;

	// Load the bitmap into the other DC
	hdcBitmap = CreateCompatibleDC(hdc);
	hbmOld = SelectObject(hdcBitmap, yi->hbmYak);

	// Prepare to create the volume window
	hbrBlack = CreateSolidBrush(COLOR_BLACK);
	hpen = CreatePen(PS_SOLID, 1, COLOR_BLACK);

	GetClientRect(hwnd, &rc);
	hbrOld = SelectObject(hdc, hbrBlack);
	hpOld = SelectObject(hdc, hpen);
	SetBkMode(hdc, TRANSPARENT);
	SetTextColor(hdc, COLOR_WHITE);
	FillRect(hdc, &rc, hbrBlack);

	// First, give the window cool rounded edges
	if (yi->fTranslucentWindows)
		RoundWindowCorners(rc, hdc, COLOR_TRANSPARENT);

	// Draw the yak on-screen
	BitBlt(hdc, NOTIFY_WND_X - YAK_PIC_X, 0, YAK_PIC_X, YAK_PIC_Y,
		hdcBitmap, 0, 0, SRCCOPY);

	// Now draw the text
	GetClientRect(hwnd, &rc);
	rc.left += 10; // margin

	// Draw top line of text
	rc.top += 10;
	SelectObject(hdc, yi->hfNotifyBold);
	GetTimeFormat(LOCALE_USER_DEFAULT, TIME_NOSECONDS, NULL, NULL, szTime, 64);
	
	if (g_fCallerID)
	{
		LoadString(yi->hinst, IDS_PHONERINGING, szString, 256);
		_snprintf(szText, 128, szString, szTime);
	}
	else
	{
		LoadString(yi->hinst, IDS_MESSAGEFORYOU, szString, 256);
		_snprintf(szText, 128, szString, szTime);
	}

	DrawText(hdc, szText, -1, &rc, DT_SINGLELINE | DT_NOPREFIX);

	// Draw name and number lines
	if (g_fCallerID)
	{
		rc.top += 27;
		SelectObject(hdc, yi->hfNotify);
		
		LoadString(yi->hinst, IDS_NAMEFMT, szString, 256);
		_snprintf(szText, 128, szString, g_szName);
		DrawText(hdc, szText, -1, &rc, DT_SINGLELINE | DT_NOPREFIX);

		rc.top += 20;
		LoadString(yi->hinst, IDS_NUMFMT, szString, 256);
		_snprintf(szText, 128, szString, g_szNumber);
		DrawText(hdc, szText, -1, &rc, DT_SINGLELINE | DT_NOPREFIX);
	}
	else
	{
		rc.top += 23;
		rc.right -= (YAK_PIC_X + NOTIFY_RIGHT_MARGIN);
		rc.bottom -= 10;
		
		if (g_fInWindow)
		{
			SelectObject(hdc, yi->hfNotifyUnderline);
			SetTextColor(hdc, COLOR_LINK);
		}
		else
			SelectObject(hdc, yi->hfNotify);

		DrawText(hdc, g_szNoCRLFText, -1, &rc, DT_END_ELLIPSIS | DT_NOPREFIX |
			DT_EDITCONTROL | DT_WORDBREAK);
	}

	// Clean up GDI objects
	SelectObject(hdcBitmap, hbmOld);
	DeleteDC(hdcBitmap);

	SelectObject(hdc, hbrOld);
	DeleteObject(hbrBlack);

	SelectObject(hdc, hpOld);
	DeleteObject(hpen);
}


void RoundWindowCorners(RECT rc, HDC hdc, COLORREF cr)
{
	int x, y;

	// Start with left side
	x = rc.left;

	for (y = rc.top; y < rc.top + 5; y++)
		SetPixelV(hdc, x, y, cr);
	
	for (y = rc.bottom; y > rc.bottom - 6; y--)
		SetPixelV(hdc, x, y, cr);
	
	for (y = rc.top; y < rc.top + 3; y++)
		SetPixelV(hdc, x + 1, y, cr);

	for (y = rc.bottom; y > rc.bottom - 4; y--)
		SetPixelV(hdc, x + 1, y, cr);

	for (y = rc.top; y < rc.top + 2; y++)
		SetPixelV(hdc, x + 2, y, cr);

	for (y = rc.bottom; y > rc.bottom - 3; y--)
		SetPixelV(hdc, x + 2, y, cr);

	y = rc.top;
	SetPixelV(hdc, x + 3, y, cr);	
	SetPixelV(hdc, x + 4, y, cr);

	y = rc.bottom - 1;
	SetPixelV(hdc, x + 3, y, cr);
	SetPixelV(hdc, x + 4, y, cr);

	// Now do right side
	x = rc.right - 1;

	for (y = rc.top; y < rc.top + 5; y++)
		SetPixelV(hdc, x, y, cr);
	
	for (y = rc.bottom; y > rc.bottom - 6; y--)
		SetPixelV(hdc, x, y, cr);
	
	for (y = rc.top; y < rc.top + 3; y++)
		SetPixelV(hdc, x - 1, y, cr);

	for (y = rc.bottom; y > rc.bottom - 4; y--)
		SetPixelV(hdc, x - 1, y, cr);

	for (y = rc.top; y < rc.top + 2; y++)
		SetPixelV(hdc, x - 2, y, cr);

	for (y = rc.bottom; y > rc.bottom - 3; y--)
		SetPixelV(hdc, x - 2, y, cr);

	y = rc.top;
	SetPixelV(hdc, x - 3, y, cr);	
	SetPixelV(hdc, x - 4, y, cr);

	y = rc.bottom - 1;
	SetPixelV(hdc, x - 3, y, cr);
	SetPixelV(hdc, x - 4, y, cr);
}


void LaunchReadTextDialog(HWND hwnd, YACINFO *yi)
{
	TCHAR szTime[64], szFormat[256], szText[256];

	if (yi->hwndReadText)
	{
		// Remove the previous message if it's still up
		DestroyWindow(yi->hwndReadText);
		yi->hwndReadText = NULL;
	}

	yi->hwndReadText = CreateDialog(yi->hinst, MAKEINTRESOURCE(IDD_READTEXT),
		yi->hwnd, ReadTextDialogProc);

	SetWindowText(GetDlgItem(yi->hwndReadText, IDC_TEXT), g_szText);
	
	// Set the title bar caption
	LoadString(yi->hinst, IDS_MSGCAPTION, szFormat, 256);
	GetTimeFormat(LOCALE_USER_DEFAULT, TIME_NOSECONDS, NULL, NULL, szTime, 64);
	sprintf(szText, szFormat, szTime);
	SetWindowText(yi->hwndReadText, szText);
}


INT_PTR CALLBACK ReadTextDialogProc(HWND hwndDlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
	YACINFO *yi;

	GetYACInfoStruct(&yi);

	switch (msg)
	{	
	case WM_CTLCOLORDLG:
		return (INT_PTR)yi->hbrBlack;

	case WM_CTLCOLORSTATIC:
		SetBkMode((HDC)wparam, TRANSPARENT);
		SetTextColor((HDC)wparam, COLOR_WHITE);
		SelectObject((HDC)wparam, yi->hfNotify);
		return (INT_PTR)GetStockObject(NULL_BRUSH);

	case WM_CTLCOLORBTN:
		SetBkMode((HDC)wparam, TRANSPARENT);
		return (INT_PTR)GetStockObject(NULL_BRUSH);

	case WM_DESTROY:
		yi->hwndReadText = NULL;
		return 0;

	case WM_COMMAND:
		switch (LOWORD(wparam))
        {
		case IDOK:
			DestroyWindow(hwndDlg);			
			break;
		}
		break;
	}

	return FALSE;
}