/*  YAC: Yet Another Caller ID Program
 *  Copyright (C) 2002 Jensen Harris (jensen@sunflowerhead.com)
 *  http://www.sunflowerhead.com
 *
 *  YAC is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any
 *  later version.
 *
 *  YAC is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 *  details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with YAC; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  File: Systray.c
 *  This file contains code to manage the notifications area icon on the
 *  taskbar, including the menu that pops up when you click on it.
 */

#include "yac.h"

BOOL InitSystemTray(YACINFO *yi)
{
	WORD wIcon;
	TCHAR szText[256], szText2[256];

	// Bugs in earlier shells mean that only XP and above get the nice icon.
	if (yi->uOS == OS_WINXP_ABOVE)
		wIcon = IDI_APP;
	else
		wIcon = IDI_SYSTRAY;

	yi->hicon = (HICON)LoadImage(yi->hinst, MAKEINTRESOURCE(wIcon),
		IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR | LR_SHARED);
	
	// Set up our tray icon information
	ZeroMemory(&yi->ni, sizeof(yi->ni));

	yi->ni.cbSize = sizeof(yi->ni);
	yi->ni.hWnd = yi->hwnd;
	yi->ni.uVersion = NOTIFYICON_VERSION;
	yi->ni.uCallbackMessage = YAC_MSG_TRAY;
	yi->ni.uTimeout = yi->nDisplayTime;
	yi->ni.dwInfoFlags = NIIF_INFO;

	// Set tooltip for icon
	LoadString(yi->hinst, IDS_APPFULLNAME, szText, 256);

	if (yi->nMode == MODE_SERVER)
		LoadString(yi->hinst, IDS_SERVER, szText2, 256);
	else
		LoadString(yi->hinst, IDS_LISTENER, szText2, 256);

	sprintf(yi->ni.szTip, "%s - %s", szText, szText2);

	yi->ni.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
	yi->ni.hIcon = yi->hicon;
	yi->ni.uID = 0;

	Shell_NotifyIcon(NIM_SETVERSION, &yi->ni);	
	Shell_NotifyIcon(NIM_ADD, &yi->ni);

	return TRUE;
}


void ShowContextMenu(HWND hwnd, YACINFO *yi)
{
	HMENU hmenu, hSubMenu;
	POINT pt;
	UINT uCheck;

	GetCursorPos(&pt);

	hmenu = LoadMenu(yi->hinst, MAKEINTRESOURCE(IDR_MENU));
	hSubMenu = GetSubMenu(hmenu, 0); 

	if (yi->fAboutDlgShowing || yi->fListenersDlgShowing ||
		yi->fDevicesDlgShowing || yi->fSubsDlgShowing ||
		yi->fFormatsDlgShowing || yi->fSendTextDlgShowing)
		EnableMenuItem(hSubMenu, IDM_EXIT, MF_GRAYED);

	if (yi->fAboutDlgShowing)
		EnableMenuItem(hSubMenu, IDM_ABOUT, MF_GRAYED);

	if (yi->fListenersDlgShowing)
		EnableMenuItem(hSubMenu, IDM_LISTENERS, MF_GRAYED);

	if (yi->fSubsDlgShowing)
		EnableMenuItem(hSubMenu, IDM_SUBSTITUTIONS, MF_GRAYED);

	if (yi->fFormatsDlgShowing)
		EnableMenuItem(hSubMenu, IDM_FORMATS, MF_GRAYED);

	if (yi->fSendTextDlgShowing)
		EnableMenuItem(hSubMenu, IDM_SENDTEXT, MF_GRAYED);

	uCheck = yi->fStartupSelectDevice ? MF_CHECKED | MF_BYCOMMAND :
		MF_UNCHECKED | MF_BYCOMMAND;
	CheckMenuItem(hSubMenu, IDM_DEVICES, uCheck);

	uCheck = yi->fTranslucentWindows ? MF_CHECKED | MF_BYCOMMAND :
		MF_UNCHECKED | MF_BYCOMMAND;
	CheckMenuItem(hSubMenu, IDM_TRANSLUCENT, uCheck);

	uCheck = yi->fUseBalloons ? MF_CHECKED | MF_BYCOMMAND :
		MF_UNCHECKED | MF_BYCOMMAND;
	CheckMenuItem(hSubMenu, IDM_USEBALLOONS, uCheck);

	uCheck = yi->fShareModem ? MF_CHECKED | MF_BYCOMMAND :
		MF_UNCHECKED | MF_BYCOMMAND;
	CheckMenuItem(hSubMenu, IDM_SHAREMODEM, uCheck);

	uCheck = (yi->fPlaySounds && !yi->fUseBalloons) ?
		MF_CHECKED | MF_BYCOMMAND : MF_UNCHECKED | MF_BYCOMMAND;
	CheckMenuItem(hSubMenu, IDM_PLAYSOUNDS, uCheck);

	uCheck = yi->fWriteLogFile ? MF_CHECKED | MF_BYCOMMAND :
		MF_UNCHECKED | MF_BYCOMMAND;
	CheckMenuItem(hSubMenu, IDM_WRITELOG, uCheck);

	if (yi->fWriteLogFile)
	{
		uCheck = yi->fWriteTextToLog ? MF_CHECKED | MF_BYCOMMAND :
			MF_UNCHECKED | MF_BYCOMMAND;
		CheckMenuItem(hSubMenu, IDM_TEXTTOLOG, uCheck);
	}
	else
	{
		EnableMenuItem(hSubMenu, IDM_TEXTTOLOG, MF_GRAYED);
	}

	if (yi->uOS < OS_WIN2000)
		EnableMenuItem(hSubMenu, IDM_TRANSLUCENT, MF_GRAYED);

	if (yi->uOS < OS_WINME)
		EnableMenuItem(hSubMenu, IDM_USEBALLOONS, MF_GRAYED);

	if (yi->fUseBalloons)
		EnableMenuItem(hSubMenu, IDM_PLAYSOUNDS, MF_GRAYED);

	if (!yi->fWriteLogFile)
		EnableMenuItem(hSubMenu, IDM_OPENLOG, MF_GRAYED);

	if (yi->nMode == MODE_LISTENER)
	{
		EnableMenuItem(hSubMenu, IDM_SHAREMODEM, MF_GRAYED);
		EnableMenuItem(hSubMenu, IDM_DEVICES, MF_GRAYED);
		EnableMenuItem(hSubMenu, IDM_LISTENERS, MF_GRAYED);
		EnableMenuItem(hSubMenu, IDM_SUBSTITUTIONS, MF_GRAYED);
		EnableMenuItem(hSubMenu, IDM_FORMATS, MF_GRAYED);
		EnableMenuItem(hSubMenu, IDM_SENDTEXT, MF_GRAYED);
	}

#ifdef _DEBUG
	if (yi->nMode == MODE_LISTENER)
		EnableMenuItem(hSubMenu, IDM_DEBUG_TESTINCOMING, MF_GRAYED);
	else if (yi->nMode == MODE_SERVER)
		EnableMenuItem(hSubMenu, IDM_DEBUG_TESTNETWORK, MF_GRAYED);
#else
	RemoveMenu(hSubMenu, IDM_DEBUG_TESTINCOMING, MF_BYCOMMAND);
	RemoveMenu(hSubMenu, IDM_DEBUG_TESTNETWORK, MF_BYCOMMAND);
#endif

	SetForegroundWindow(hwnd);
	TrackPopupMenuEx(hSubMenu, TPM_LEFTALIGN | TPM_BOTTOMALIGN | TPM_RIGHTBUTTON,
		pt.x, pt.y, hwnd, NULL);
	DestroyMenu(hSubMenu);
	DestroyMenu(hmenu);
}