/*  YAC: Yet Another Caller ID Program
 *  Copyright (C) 2002 Jensen Harris (jensen@sunflowerhead.com)
 *  http://www.sunflowerhead.com
 *
 *  YAC is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any
 *  later version.
 *
 *  YAC is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 *  details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with YAC; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  File: Utility.c
 *  General utility functions needed throughout YAC.
 */

#include "yac.h"

BOOL GetProcessFolder(TCHAR *szFolder)
{
	HMODULE hm = GetModuleHandle(NULL);
	if (0 == GetModuleFileName(hm, szFolder, MAX_PATH))
		return FALSE;

	PathRemoveFileSpec(szFolder);

	return TRUE;
}


void NotifyError(UINT uiString, BOOL fShowLastError, HINSTANCE hinst, YACINFO *yi)
{
	TCHAR szText[256];
	LPVOID lpMsgBuf;

	LoadString(hinst, uiString, szText, 256);

	if (fShowLastError)
	{
		FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | 
			FORMAT_MESSAGE_IGNORE_INSERTS, NULL, GetLastError(),
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) &lpMsgBuf, 0, NULL);
		strncat(szText, "\n\n", 256);
		strncat(szText, lpMsgBuf, 256);

		LocalFree(lpMsgBuf);
	}

	MessageBox(GetDesktopWindow(), szText, yi->szAppName, MB_ICONSTOP | MB_OK);
}


void GetIniFileName(TCHAR *szName)
{
	GetProcessFolder(szName);
	strncat(szName, "\\yac.ini", MAX_PATH);
}


void GetLogFileName(TCHAR *szName)
{
	GetProcessFolder(szName);
	strncat(szName, "\\yac-log.txt", MAX_PATH);
}


BOOL GetInfo(LPCSTR lpcstrKey, TCHAR *szSetting, HKEY hkHive)
{
	HKEY hkey;
	LONG lRet;
	DWORD dType, dSize;

	lRet = RegCreateKey(
		hkHive,
		APPREGKEY,
		&hkey
		);

	lRet = RegQueryValueEx(
		hkey,
		lpcstrKey,
		NULL,
		&dType,
		NULL,
		&dSize
		);

	if ((ERROR_SUCCESS != lRet) || (dSize > MAX_PATH))
	{
		RegCloseKey(hkey);
		return FALSE;
	}

	lRet = RegQueryValueEx(
		hkey,
		lpcstrKey,
		NULL,
		&dType,
		(unsigned char*)szSetting, 
		&dSize
		);

	lRet = RegCloseKey(hkey);

	return TRUE;
}


void SaveInfo(LPCSTR lpcstrKey, LPCSTR lpcstrSetting, HKEY hkHive)
{
	HKEY hkey;
	LONG lRet;
	const unsigned char *cucszSetting;

	cucszSetting = (const unsigned char*)lpcstrSetting;

	lRet = RegCreateKey(
		hkHive,
		APPREGKEY,
		&hkey 
		);

	lRet = RegSetValueEx(
		hkey,
		lpcstrKey,
		0,
		REG_SZ,
		cucszSetting,
		(DWORD)strlen(lpcstrSetting)  
		);

	lRet = RegCloseKey(hkey);

	return;
}


void WriteLog(TCHAR *szText, YACINFO *yi)
{
	HANDLE hf;
	DWORD dwWritten;

	if (!yi->fWriteLogFile)
		return;

	hf = CreateFile(
		yi->szLogFile,
		GENERIC_WRITE,
		FILE_SHARE_READ,
		NULL,
		OPEN_ALWAYS,
		FILE_ATTRIBUTE_NORMAL,
		NULL);

	if (hf == INVALID_HANDLE_VALUE)
		return;

	SetFilePointer (hf, 0, NULL, FILE_END);
	WriteFile(hf, szText, (DWORD)strlen(szText), &dwWritten, NULL);

	CloseHandle(hf);
}


void WriteLogIncomingCall(TCHAR *szName, TCHAR *szNumber, YACINFO *yi)
{
	TCHAR szText[128], szLogText[256], szFormat[256];

	GetDateTimeString(szText, 128);
	
	LoadString(yi->hinst, IDS_INCOMING_CALL_HEADER, szFormat, 256);
	_snprintf(szLogText, 256, szFormat, szText);
	WriteLog(szLogText, yi);

	LoadString(yi->hinst, IDS_LOGFILE_NAME, szFormat, 256);
	_snprintf(szLogText, 256, szFormat, szName);
	WriteLog(szLogText, yi);

	LoadString(yi->hinst, IDS_LOGFILE_NUMBER, szFormat, 256);
	_snprintf(szLogText, 256, szFormat, szNumber);
	WriteLog(szLogText, yi);
}


void WriteLogIncomingMessage(TCHAR *szInText, YACINFO *yi)
{
	TCHAR szText[128], szLogText[256], szFormat[256];

	GetDateTimeString(szText, 128);
	
	LoadString(yi->hinst, IDS_INCOMING_MSG_HEADER, szFormat, 256);
	_snprintf(szLogText, 256, szFormat, szText);
	WriteLog(szLogText, yi);
	WriteLog(szInText, yi);
	
	LoadString(yi->hinst, IDS_CRLF2, szFormat, 256);
	WriteLog(szFormat, yi);
}


void GetDateTimeString(TCHAR *szOut, UINT cchMax)
{
	TCHAR szDate[128], szTime[64];
	
	GetTimeFormat(LOCALE_USER_DEFAULT, 0, NULL, NULL, szTime, 64);
	GetDateFormat(LOCALE_USER_DEFAULT, DATE_LONGDATE, NULL, NULL, szDate, 128);

	_snprintf(szOut, cchMax, "%s - %s", szDate, szTime);
}


HFONT GetFont(TCHAR *szFaceName, BOOL fBold, BOOL fItalic, BOOL fUnderline,
			  int nPoint, int nWidth)
{
	LOGFONT lf;
	HDC hdc;

	hdc = GetDC(GetDesktopWindow());

	ZeroMemory(&lf, sizeof(lf));

	// Normal font metrics
	lf.lfEscapement = 0; 
	lf.lfOrientation = 0; 
	lf.lfUnderline = fUnderline; 
	lf.lfStrikeOut = FALSE; 
	lf.lfCharSet = ANSI_CHARSET; 
	lf.lfOutPrecision = OUT_DEFAULT_PRECIS; 
	lf.lfClipPrecision = CLIP_DEFAULT_PRECIS; 
	lf.lfQuality = PROOF_QUALITY | ANTIALIASED_QUALITY | CLEARTYPE_QUALITY; 
	lf.lfPitchAndFamily = DEFAULT_PITCH | FF_DONTCARE; 

	StrNCpy(lf.lfFaceName, szFaceName, LF_FACESIZE);
	lf.lfWidth = nWidth;
	lf.lfHeight = -MulDiv(nPoint, GetDeviceCaps(hdc, LOGPIXELSY), 72);
	lf.lfItalic = fItalic;

	if (fBold)
		lf.lfWeight = FW_BOLD;
	else
		lf.lfWeight = FW_NORMAL;
	
	ReleaseDC(GetDesktopWindow(), hdc);
	return (CreateFontIndirect(&lf));
}


void EncodeCallerIDInfo(TCHAR *szOut, TCHAR *szName, TCHAR *szNumber)
{
	sprintf(szOut, "%s%s~%s", CALL_TAG, szName, szNumber);
}


BOOL DecodeCallerIDInfo(TCHAR *szInput, TCHAR *szName, TCHAR *szNumber)
{
	int ch, i;
	BOOL fCopyToNumber = FALSE, fIsCallerID = FALSE;

	ch = 0;
	szName[0] = '\0';
	szNumber[0] = '\0';

	fIsCallerID = IsCallerIDInfoAndStripTag(szInput, (DWORD)strlen(szInput));

	if (fIsCallerID)
	{
		// Parse szInput, if applicable, into name and number
		for (i = 0; i < (int)strlen(szInput); i++)
		{
			if ((szInput[i] == '~') && !fCopyToNumber)
			{
				fCopyToNumber = TRUE;
				szName[ch] = '\0';
				ch = -1;
			}
			else if (!fCopyToNumber)
			{
				szName[ch] = szInput[i];
			}
			else
			{
				szNumber[ch] = szInput[i];
			}

			ch++;
		}

		if (fCopyToNumber)
			szNumber[ch] = '\0';
		
		return TRUE;
	}
	else
		return FALSE;
}


BOOL IsCallerIDInfoAndStripTag(TCHAR *szText, DWORD cbLen)
{
	TCHAR szTemp[6], szOutText[MAX_CHAR_SENT];
	BOOL fFoundMarker;
	int i;

	// This delightful function will see if incoming text is Caller ID info,
	// stripping the tag if so.

	strncpy(szTemp, szText, 5);
	szTemp[5] = '\0';

	if (!StrCmp(CALL_TAG, szTemp))
	{
		// This is probably Caller ID info, check for a ~ character
		fFoundMarker = FALSE;

		for (i = 0; i < (int)strlen(szText); i++)
		{
			if (szText[i] == '~')
			{
				fFoundMarker = TRUE;
				break;
			}
		}

		if (fFoundMarker)
		{
			// Yep, it's definitely Caller ID info, remove the initial tag
			strncpy(szOutText, &szText[5], MAX_CHAR_SENT);
			strncpy(szText, szOutText, cbLen);
			return TRUE;
		}
	}

	return FALSE;
}


TCHAR GetRandomNumChar(int nPos)
{
	// Return a character 0-9, but if returning a first character,
	// skip 0 and 1 because they look fake in US phone numbers

	if (nPos == 0)
		return (char)(rand() % 8) + ASCII_2;

	else
		return (char)(rand() % 10) + ASCII_0;
}

void StripCRLF(TCHAR *szText)
{
	int i;

	for (i = 0; i < (int)strlen(szText); i++)
	{
		if (szText[i] == '\r' || szText[i] == '\n' || szText[i] == '\t')
			szText[i] = ' ';
	}
}