/*  YAC: Yet Another Caller ID Program
 *  Copyright (C) 2002 Jensen Harris (jensen@sunflowerhead.com)
 *  http://www.sunflowerhead.com
 *
 *  YAC is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2 of the License, or (at your option) any
 *  later version.
 *
 *  YAC is distributed in the hope that it will be useful, but WITHOUT ANY
 *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 *  details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with YAC; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  File: yac.h
 *  Version: 0.17
 *  Last Modified: 17-Apr-07 by Matt Collinge to add external script support
 */

// Defines
#define TAPI_CURRENT_VERSION	0x00020000
#define _WIN32_IE				0x0500
#define WINVER					0x0501
#define _WIN32_WINNT			0x0501

#ifndef	_WINSOCKAPI_
#define	_WINSOCKAPI_
#endif

#pragma warning(disable: 4312)

// Includes
#include <windows.h>
#include <winsock2.h>
#include <windowsx.h>
#include <commctrl.h>
#include <tapi.h>
#include <process.h>
#include <stdio.h>
#include <shlwapi.h>
#include <mmsystem.h>
#include <time.h>
#include "resource.h"

// Post-include Defines
#define VERSION					"0.17"
#define YAC_MSG_TRAY			WM_USER + 1
#define YAC_MSG_NOTIFY			WM_USER + 2
#define YAC_MSG_QUIT			WM_USER + 3
#define YAC_MSG_NOTIFYUSER		WM_USER + 4
#define PRIVATE					1
#define OUTOFAREA				2
#define UNAVAILABLE				3
#define MSG_WAIT_TIME			500
#define APPREGKEY				"Software\\YAC"
#define NONESTRING				"~~none~~"
#define MODE_SERVER				1
#define MODE_LISTENER			2
#define MAX_CHAR_SENT			400
#define MSG_LIMIT_CHARS			300
#define MAX_DEVICES				32
#define COLOR_TRANSPARENT		RGB(3, 4, 5)
#define COLOR_BLACK				RGB(0, 0, 0)
#define COLOR_WHITE				RGB(255, 255, 255)
#define COLOR_LINK				RGB(255, 255, 153)
#define MENU_TRANSPARENCY		215
#define TIMER_NOTIFYWND			1
#define YAK_PIC_X				71
#define YAK_PIC_Y				91
#define NOTIFY_WND_X			300
#define NOTIFY_WND_Y			YAK_PIC_Y
#define NOTIFY_RIGHT_MARGIN		10
#define PHONE_COLUMN_SIZE		125
#define NUMBER_COLUMN_SIZE		50
#define ASCII_0					48
#define ASCII_2					50
#define SUPPORTED_FORMATS		20
#define SERVER_NAME				"YAC"
#define LISTENER_NAME			"YAC Listener"
#define YAC_WND_CLASS			"YACMainWindow"
#define YAC_NOTIFYWND_CLASS		"YACNotifyWindow"
#define YAC_MUTEX				"YAC_MUTEX"
#define YAC_LINE_IDENT			"YAC: Yet Another Caller ID Program"
#define INI_FORMATS				"Formats"
#define INI_SUBS				"Substitutions"
#define INI_LISTENERS			"Listeners"
#define CALL_TAG				"@CALL"
#define DEFAULT_7_FORMAT		"###-####"
#define DEFAULT_10_FORMAT		"(###) ###-####"
#define TEST_NAME				"YAC Test Call"
#define TEST_NUMBER				"4255551212"
#define TEST_NUMBER_FMT			"(425) 555-1212"
#define IDC_EDIT_SHOWTEXT		4750
#define DEFAULT_DISPLAY_TIME	20000
#define MIN_DISPLAY_TIME		1000
#define MAX_DISPLAY_TIME		300000
#define DEFAULT_PORT			10629
#define URL_HOMEPAGE			"http://www.sunflowerhead.com/software/yac/"
#define URL_EMAIL				"mailto:jensen@sunflowerhead.com"
#define URL_SUPPORT				"http://www.tivocommunity.com/tivo-vb/forumdisplay.php?s=&forumid=8"
#define URL_LICENSE				"http://www.gnu.org/licenses/gpl.txt"
#define URL_NEWEST				"http://www.sunflowerhead.com/software/yac/index.html#newest"
#define URL_LISTENERS			"http://www.sunflowerhead.com/software/yac/index.html#listeners"
#define URL_SOURCE				"http://www.sunflowerhead.com/software/yac/index.html#source"
#define URL_LEARNYAK			"http://www.google.com/search?q=yaks"

// The relative positions of the OS's do mean something.
// Don't change their order unless you know what you're doing.
#define OS_NOT_SUPPORTED		0
#define OS_WIN95				1
#define OS_NT4					2
#define OS_WIN98				3
#define OS_WINME				4
#define OS_WIN2000				5
#define OS_WINXP_ABOVE			6

// Structures
typedef struct {
	TCHAR	szDevice[64];
	int		nListBoxID;
} DEVICEINFO;

typedef BOOL (WINAPI* SETLAYEREDWINDOWATTRIBUTESFUNC)(HWND, COLORREF, BYTE, DWORD); 

typedef struct
{
	BOOL			fAboutDlgShowing;
	BOOL			fDevicesDlgShowing;
	BOOL			fListenersDlgShowing;
	BOOL			fSubsDlgShowing;
	BOOL			fFormatsDlgShowing;
	BOOL			fSendTextDlgShowing;
	BOOL			fStartupSelectDevice;
	BOOL			fTranslucentWindows;
	BOOL			fUseBalloons;
	BOOL			fShareModem;
	BOOL			fPlaySounds;
	BOOL			fWriteLogFile;
	BOOL			fWriteTextToLog;

	DEVICEINFO		DevInfo[MAX_DEVICES];

	DWORD			dwNumDevs;

	HANDLE			Mutex;
	HANDLE			EventNotify;
	HANDLE			EventDisplayQuit;
	HANDLE			EventPhoneListenQuit;
	HANDLE			EventNotifyNetQuit;
	HANDLE			EventNetNotify;
	HANDLE			EventListenQuit;
	HANDLE			hDisplayThread;
	HANDLE			hPhoneThread;
	HANDLE			hNotifyNetThread;
	HANDLE			hListenThread;

	HBITMAP			hbmYak;

	HBRUSH			hbrBlack;

	HCURSOR			hcHand;
	HCURSOR			hcArrow;

	HFONT			hfBold;
	HFONT			hfNotify;
	HFONT			hfNotifyUnderline;
	HFONT			hfNotifyBold;

	HICON			hicon;

	HIMAGELIST		im;
	HINSTANCE		hinst;

	HLINEAPP		hlapp;

	HWND			hwnd;
	HWND			hwndNotify;
	HWND			hwndReadText;

	int				nNumListeners;
	int				nMode;
	int				nDisplayTime;
	int				nServerPort;

	LPCSTR			szIncomingSound;
	LPCSTR			szDoneSound;

	NOTIFYICONDATA	ni;

	SOCKET			serverFd;

	TCHAR			szIniFile[MAX_PATH];
	TCHAR			szLogFile[MAX_PATH];
	TCHAR			szAddr[10][256];
	TCHAR			szAppName[32];
	TCHAR			szName[32];
	TCHAR			szNumber[32];
	TCHAR			szString[MAX_CHAR_SENT];

	UINT			uTaskbarReset;
	UINT			uOS;
} YACINFO;


// Prototypes
INT_PTR CALLBACK	AboutDialogProc(HWND hwndDlg, UINT msg, WPARAM wparam, LPARAM lparam);
void				AddNewItem(HWND hwnd, YACINFO *yi);
void				CalcExampleText(TCHAR *szText, int nNumDigits);
void				CleanUpResources(YACINFO *yi);
void				CreateGDIResources(YACINFO *yi);
HWND				CreateHiddenWindow(YACINFO *yi);
HWND				CreateNotifyWindow(YACINFO *yi);
BOOL				DecodeCallerIDInfo(TCHAR *szOut, TCHAR *szName, TCHAR *szNumber);
INT_PTR CALLBACK	DevicesDialogProc(HWND hwndDlg, UINT msg, WPARAM wparam, LPARAM lparam);
unsigned __stdcall	DisplayMessageThread(void* lpParameter);
unsigned __stdcall	DisplayNotifyWindowThread(void* lpParameter);
void				DoFirstRun(YACINFO *yi);
void				DoFormatsEditChanged(HWND hwnd);
void				DoFormatsSelectionChanged(HWND hwnd);
BOOL				DoNameSubstitution(TCHAR *szName, TCHAR *szNumber, YACINFO *yi);
void				EncodeCallerIDInfo(TCHAR *szOut, TCHAR *szName, TCHAR *szNumber);
BOOL				EnsureNotRunning(HINSTANCE hinst);
SOCKET				EstablishConnection(unsigned long nRemoteAddr, unsigned short nPort);
void				ExecuteHelperScript(); // MattC 17-Apr-07
void				FindAndDestroyNotifyWindow();
void				FormatNameAndNumber(TCHAR *szName, TCHAR *szNumber, YACINFO *yi);
INT_PTR CALLBACK	FormatsDialogProc(HWND hwndDlg, UINT msg, WPARAM wparam, LPARAM lparam);
HFONT				GetFont(TCHAR *szFaceName, BOOL fBold, BOOL fItalic, BOOL fUnderline, int nPoint, int nWidth);
void				GetDateTimeString(TCHAR *szOut, UINT cchMax);
BOOL				GetInfo(LPCSTR lpcstrKey, TCHAR *szSetting, HKEY hkHive);
void				GetIniFileName(TCHAR *szName);
void				GetLogFileName(TCHAR *szName);
BOOL				GetProcessFolder(TCHAR *szFolder);
TCHAR				GetRandomNumChar(int nPos);
void				GetRegistryOptions(YACINFO *yi);
BOOL				GetYACInfoStruct(YACINFO **yi);
void				GoAboutCommand(HWND hwnd, YACINFO *yi);
void				InitAboutDialog(HWND hwnd, YACINFO *yi);
void				InitDevicesDialog(HWND hwnd, YACINFO *yi);
BOOL				InitFormatsDialog(HWND hwnd, YACINFO *yi);
void				HandleCopyData(HWND hwnd, WPARAM wparam, LPARAM lparam, YACINFO *yi);
void				InitYACInfoStruct(YACINFO *yi);
void				InitListenersDialog(HWND hwnd, YACINFO *yi);
void				InitLogFile(YACINFO *yi);
BOOL				InitSubsDialog(HWND hwnd, YACINFO *yi);
BOOL				InitSystemTray(YACINFO *yi);
BOOL				InitYAC(LPSTR lpCmdLine, YACINFO *yi);
BOOL				IsCallerIDInfoAndStripTag(TCHAR *szText, DWORD cbLen);
void				LaunchReadTextDialog(HWND hwnd, YACINFO *yi);
INT_PTR CALLBACK	ListenersDialogProc(HWND hwndDlg, UINT msg, WPARAM wparam, LPARAM lparam);
unsigned __stdcall	ListenThread(void* lpParameter);
u_long				LookupAddress(const char* pcHost);
LRESULT CALLBACK	MainWndProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam);
void				MakeRawNumber(TCHAR *szNumber);
unsigned __stdcall	NetConnectThread(void* lpParameter);
void				NotifyClients(YACINFO *yi);
void				NotifyError(UINT uiString, BOOL fShowLastError, HINSTANCE hinst, YACINFO *yi);
void				NotifyNetworkClients();
BOOL				NotifyUser(TCHAR *szText, YACINFO *yi);
LRESULT CALLBACK	NotifyWndProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam);
void				PaintNotifyWindow(HWND hwnd, HDC hdc, YACINFO *yi);
unsigned __stdcall	PhoneListenThread(void* lpParameter);
void				ReadLineFromSocket(SOCKET fd, char *szText);
void				ReadListenersData(YACINFO *yi);
INT_PTR CALLBACK	ReadTextDialogProc(HWND hwndDlg, UINT msg, WPARAM wparam, LPARAM lparam);
void				RemoveSelectedItem(HWND hwnd, YACINFO *yi);
void				RoundWindowCorners(RECT rc, HDC hdc, COLORREF cr);
INT_PTR CALLBACK	SendTextDialogProc(HWND hwndDlg, UINT msg, WPARAM wparam, LPARAM lparam);
void				SaveFormatsData(HWND hwnd, YACINFO *yi);
void				SaveInfo(LPCSTR lpcstrKey, LPCSTR lpcstrSetting, HKEY hkHive);
void				SaveListenersData(HWND hwnd, YACINFO *yi);
void				SendNotificationString(YACINFO *yi);
void				SendText(HWND hwnd, YACINFO *yi);
void				SetUpMouseTracking(HWND hwnd);
void				ShowContextMenu(HWND hwnd, YACINFO *yi);
BOOL				ShowShareModemWarning(YACINFO *yi);
void				StopNotifyingUser(YACINFO *yi);
void				StripCRLF(TCHAR *szText);
INT_PTR CALLBACK	SubsDialogProc(HWND hwndDlg, UINT msg, WPARAM wparam, LPARAM lparam);
void				TestListeners(HWND hwnd, YACINFO *yi);
BOOL				ValidateOperatingSystem(YACINFO *yi);
BOOL				VerifyExit(YACINFO *yi);
void				WriteLog(TCHAR *szText, YACINFO *yi);
void				WriteLogIncomingCall(TCHAR *szName, TCHAR *szNumber, YACINFO *yi);
void				WriteLogIncomingMessage(TCHAR *szText, YACINFO *yi);